
import rtdl
import zero
import torch
import torch.nn as nn
from tab_transformer_pytorch import TabTransformer
import torch.utils.data as D
from train_funct import train, val, test
import numpy as np 
import time 
import torch
import matplotlib.pyplot as plt 
import random

def seed_everything(seed: int):
    r"""Sets the seed for generating random numbers in PyTorch, numpy and
    Python.

    Args:
        seed (int): The desired seed.
    """
    random.seed(seed)
    np.random.seed(seed)
    torch.manual_seed(seed)
    torch.cuda.manual_seed_all(seed)

if __name__ == '__main__':
    # Params
    device = 'cuda' if torch.cuda.is_available() else 'cpu'
    lr = 0.00005
    batch_size = 64
    n_epochs= 80
    seed = 1
    seed_everything(seed)
    
    test_AUC, test_PR_AUC, test_ACC, test_BA, test_MCC, test_CK, test_sensitivity, test_specificity, test_precision, test_F1 = [], [], [], [], [], [], [], [], [], []
    
    # Load datas
    train_cats_data = torch.LongTensor(np.load('./input_data/process_data/train_cats.npy'))
    train_nums_data = torch.LongTensor(np.load('./input_data/process_data/train_nums.npy'))
    train_labels = torch.LongTensor(np.load('./input_data/process_data/train_labels.npy'))
    train_datasets = D.TensorDataset(train_cats_data, train_nums_data, train_labels)
    train_dataloader = D.DataLoader(train_datasets,batch_size, shuffle=True)

    val_cats_data = torch.LongTensor(np.load('./input_data/process_data/val_cats.npy'))
    val_nums_data = torch.LongTensor(np.load('./input_data/process_data/val_nums.npy'))
    val_labels = torch.LongTensor(np.load('./input_data/process_data/val_labels.npy'))
    val_datasets = D.TensorDataset(val_cats_data, val_nums_data, val_labels)
    val_dataloader = D.DataLoader(val_datasets, batch_size, shuffle=False)

    test_cats_data = torch.LongTensor(np.load('./input_data/process_data/test_cats.npy'))
    test_nums_data = torch.LongTensor(np.load('./input_data/process_data/test_nums.npy'))
    test_labels = torch.LongTensor(np.load('./input_data/process_data/test_labels.npy'))
    test_datasets = D.TensorDataset(test_cats_data, test_nums_data, test_labels)
    test_dataloader = D.DataLoader(test_datasets, batch_size, shuffle=False)

    cont_mean_std = torch.randn(train_nums_data.shape[1], 2)
    model = TabTransformer(
        categories = (3, 3, 5, 2, 9, 3, 1, 2),      # tuple containing the number of unique values within each category
        num_continuous = train_nums_data.shape[1],                # number of continuous values
        dim = 32,                           
        dim_out = 1,                       
        depth = 3,                        
        heads = 5,                         
        attn_dropout = 0.1,              
        ff_dropout = 0.1,                
        mlp_hidden_mults = (4, 2),         
        mlp_act = nn.ReLU(),                
        continuous_mean_std = cont_mean_std
    )
    model.to(device)
    optimizer = torch.optim.AdamW(model.parameters(), lr=lr)
    loss_fn =   torch.nn.BCELoss()


    # Training 
    train_loss_list = []
    val_loss_list = []
    val_loss_check = 10
    for epoch in range(n_epochs):
        loss_train =  train(epoch, model, optimizer, train_dataloader, loss_fn, device)
        loss_val   =  val(epoch, model, val_dataloader, loss_fn, device)
        train_loss_list.append(loss_train)
        val_loss_list.append(loss_val)
        if loss_val <= val_loss_check:
            print(' <<< BEST VALIDATION EPOCH')
            test_perform = test(epoch, model, test_dataloader, loss_fn, device)
            val_loss_check = loss_val
            torch.save(model.state_dict(), f'./output/model.pth')
    test_AUC.append(test_perform[0])
    test_PR_AUC.append(test_perform[1])
    test_ACC.append(test_perform[2])
    test_BA.append(test_perform[3])
    test_MCC.append(test_perform[4])
    test_CK.append(test_perform[5])
    test_sensitivity.append(test_perform[6])
    test_specificity.append(test_perform[7])
    test_precision.append(test_perform[8])
    test_F1.append(test_perform[9])
    x = np.arange(1,len(train_loss_list)+1)
    plt.plot(x, train_loss_list, '-b', label='Training Loss')
    plt.plot(x, val_loss_list, '-r', label='Validation Loss')
    plt.xticks(np.arange(1, len(train_loss_list) +1, step=10))
    plt.xlabel("Epoch")
    plt.ylabel("Loss")
    plt.legend(loc='upper right')
    # plt.title(title)

    # save image
    plt.savefig(f"./output/loss.pdf")  # should before show method
    plt.close()

# save result test
import pandas as pd 
df = pd.DataFrame({
            'test_AUC'        : test_AUC,
            'test_PR_AUC'     : test_PR_AUC,
            'test_ACC'        : test_ACC,
            'test_BA'         : test_BA,
            'test_MCC'        : test_MCC,
            'test_CK'         : test_CK,
            'test_sensitivity': test_sensitivity,
            'test_specificity': test_specificity,
            'test_precision'  : test_precision,
            'test_F1'         : test_F1})

df.to_csv(f'./output/perform_test.csv', index=False)

